'use strict';

global.__base = __dirname + '/../';


var test  = require('unit.js');
var vm = require("vm");
var fs = require("fs");

var load = function(path, context) {
  context = context || {};
  var data = fs.readFileSync(path);
  vm.runInNewContext(data, context, path);
  return context;
}

function node(hostname, uniqueKey) {
  this.name = hostname;
  this.uniqueKey = uniqueKey;
}

node.prototype.get = function(attr) {
  if (attr === 'hostname')
    return this.name;
}

var executionScript = global.__base + 'addcephwizard.js';
var utilsScript = global.__base + 'util.js';

describe('checkOSDs', function(){
  it('should be true', function(){

    var utils = load(utilsScript);
    var context = load(executionScript, { OSDAssociation: utils.OSDAssociation});
    
    test
      .bool(context.checkOSDs()).isTrue();
  });

  it('should update the global OSD Default Pool size to 10', function() {
    var utils = load(utilsScript);
    var context = load(executionScript, { OSDAssociation: utils.OSDAssociation});
    
    context.userChoices.globalOSDPoolDefaultSize = 10;

    test
      .number(context.userChoices.globalOSDPoolDefaultSize).is(10);
  });

  
  it('should have 5 OSDs when Node1 has 2 and Node2 has 1', function() {
    var utils = load(utilsScript);
    var context = load(executionScript, { OSDAssociation: utils.OSDAssociation});
    
    context.userChoices.addContainers('OSD', 'Node', [1, 2]);

    var osds = context.userChoices.containerMap['OSD'];
    var osd1Assoc0 = osds[1].osdAssociations[0];
    osd1Assoc0.blockdev = '/dev/sdb';

    var osd1Assoc1 = new utils.OSDAssociation('osd1');
    osd1Assoc1.blockdev = '/dev/sdc';
    osds[1].osdAssociations.push(osd1Assoc1);

    var osd2Assoc1 = new utils.OSDAssociation('node2-osd1');
    osd2Assoc1.blockdev = '/dev/sdf';
    osds[2].osdAssociations.push(osd2Assoc1);

    var osd2Assoc2 = new utils.OSDAssociation('node2-osd2');
    osd2Assoc2.blockdev = '/dev/sdg';
    osds[2].osdAssociations.push(osd2Assoc2);
    
    
    test
      .number(Object.keys(context.userChoices.containerMap['OSD'][1].osdAssociations).length).is(2);
    test
      .number(Object.keys(context.userChoices.containerMap['OSD'][2].osdAssociations).length).is(3);

    
    var countOSDs = context.userChoices.countOSDs();
    test
      .number(countOSDs).is(5);
  });

  it('should have 3 nodes in null category', function() {
    var utils = load(utilsScript);
    var currentCluster = {
      findNodesIn: function(type, uniqueKey) {
        if (type === 'category' && uniqueKey === 1)
          return ['', '', ''];

        return [];
      }
    };
    
    var context = load(executionScript, {
      OSDAssociation: utils.OSDAssociation,
      current: currentCluster
    });

    context.userChoices.addContainers('OSD', 'Category', [1]);
    
    var numberOfNodes = context.userChoices.getNumberOfNodesInCategory(1);
    test.number(numberOfNodes).is(3);
  });

  it('should have 0 OSDs when no categories and no nodes are configured', function() {
    var utils = load(utilsScript);

    var currentCluster = {
      findNodesIn: function(type, uniqueKey) {
        return [];
      }
    }

    var context = load(executionScript, {
      OSDAssociation: utils.OSDAssociation,
      current: currentCluster
    });

    var countOSDs = context.userChoices.countOSDs();
    test
      .number(countOSDs).is(0);
  });

  it('should have 4 OSDs when Category default contains 2 nodes and 2 OSDAssociations', function() {
    var utils = load(utilsScript);

    var currentCluster = {
      findNodesIn: function(type, uniqueKey) {
        if (type === 'category' && uniqueKey === 1)
          return ['', ''];

        return [];
      }
    }

    var context = load(executionScript, {
      OSDAssociation: utils.OSDAssociation,
      current: currentCluster
    });
    
    context.userChoices.addContainers('OSD', 'Category', [1]);

    var osds = context.userChoices.containerMap['OSD'];
    var osd1Assoc0 = osds[1].osdAssociations[0];
    osd1Assoc0.blockdev = '/dev/sdb';

    var osd1Assoc1 = new utils.OSDAssociation('osd1');
    osd1Assoc1.blockdev = '/dev/sdc';
    osds[1].osdAssociations.push(osd1Assoc1);

    test
      .number(Object.keys(context.userChoices.containerMap['OSD'][1].osdAssociations).length).is(2);

    
    var countOSDs = context.userChoices.countOSDs();
    test
      .number(countOSDs).is(4);
  });

  it('should have 15 OSDs when Category (1) contains 4 nodes with 2 OSDAssociations and OSD#2 contains 3 associations and OSD#3 contains 4 associations', function() {
    var utils = load(utilsScript);

    var currentCluster = {
      findNodesIn: function(type, uniqueKey) {
        if (type === 'category' && uniqueKey === 1)
          return ['', '', '', ''];

        return [];
      }
    }

    var context = load(executionScript, {
      OSDAssociation: utils.OSDAssociation,
      current: currentCluster
    });
    
    context.userChoices.addContainers('OSD', 'Category', [1]);
    context.userChoices.addContainers('OSD', 'Node', [2, 3]);

    //osds for category 1
    var osds = context.userChoices.containerMap['OSD'];
    var osd1Assoc0 = osds[1].osdAssociations[0];
    osd1Assoc0.name = 'category1-osd0'
    osd1Assoc0.blockdev = '/dev/sdb';

    var osd1Assoc1 = new utils.OSDAssociation('category1-osd1');
    osd1Assoc1.blockdev = '/dev/sdc';
    osds[1].osdAssociations.push(osd1Assoc1);

    //osds for node2
    osds[2].osdAssociations.push(new utils.OSDAssociation('node2-osd1'));
    osds[2].osdAssociations.push(new utils.OSDAssociation('node2-osd2'));

    //osds for node3
    osds[3].osdAssociations.push(new utils.OSDAssociation('node3-osd1'));
    osds[3].osdAssociations.push(new utils.OSDAssociation('node3-osd2'));
    osds[3].osdAssociations.push(new utils.OSDAssociation('node3-osd3'));

    test
      .number(Object.keys(context.userChoices.containerMap['OSD'][1].osdAssociations).length).is(2);

    test
      .number(Object.keys(context.userChoices.containerMap['OSD'][2].osdAssociations).length).is(3);

    test
      .number(Object.keys(context.userChoices.containerMap['OSD'][3].osdAssociations).length).is(4);

    
    var countOSDs = context.userChoices.countOSDs();
    test
      .number(countOSDs).is(15);
  });
});

describe('OSDs inventory', function() {
  it('should not have duplicated hosts when a osd is set on a category and nodes', function() {
    var utils = load(utilsScript);

    var currentCluster = {
      findNodesIn: function(type, uniqueKey) {
        if (type === 'category' && uniqueKey === 100)
          return [1, 2, 3, 4];

        throw 'cluster::findNodesIn do not support those values :' + type + ' ' + uniqueKey;
      },

      find: function(uniqueKey) {
        if (uniqueKey === 1)
          return new node('node001', 1);
        if (uniqueKey === 2)
          return new node('node002', 2);
        if (uniqueKey === 3)
          return new node('node003', 3);
        if (uniqueKey === 4)
          return new node('node004', 4);
        if (uniqueKey === 5)
          return new node('node005', 5);

        throw 'cluster::find do not support those values :' + uniqueKey;
      }
    }

    var context = load(executionScript, {
      OSDAssociation: utils.OSDAssociation,
      current: currentCluster
    });
    
    context.userChoices.addContainers('OSD', 'Category', [100]);
    context.userChoices.addContainers('OSD', 'Node', [2, 3, 5]);

    var osdHosts = context.userChoices.getAllOSDHostnames();

    test
      .number(osdHosts.size).is(5);

    test
      .bool(osdHosts.has('node005')).isTrue();

    test
      .bool(osdHosts.has('node004')).isTrue();
  });
});
