/*
 * Copyright (c) 2004-2016 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

// Poor man's enum, use like BigDataPreconditionCheckUtil.CHECK_TARGET.HEADNODES_ONLY
var BigDataPreconditionCheckUtilTarget = {"HEADNODES_ONLY": 1, "INVOLVED_NODES" : 2};

/**
 * Preconditions checker helper class for Big Data. Currently supports checking tarballs and JAVA_HOME.
 *
 * Example usage:
 * <pre>
 * var outputObject = MultipleCheckersProgressStepDisplayer('outputObject');
 * or:
 * var outputObject = ExtendHadoopDistributionWizardDeck();
 *
 * var instructions = {
 *     'get_tarballs' : [ BigDataPreconditionCheckUtilTarget.HEADNODES_ONLY ],
 *       // results the tarballs to resultsArray, fetch these using .results()!
 *
 *     'check_tarballs' : [ BigDataPreconditionCheckUtilTarget.HEADNODES_ONLY,
 *                          [ '/path/to/tarball1.tar.gz', '/path/to/tarball2.tar.gz' ]
 *                        ],
 *     'check_javahome' : [ BigDataPreconditionCheckUtilTarget.INVOLVED_NODES,
 *                          '/path/to/jvm/java-170/'
 *                        ],
 *     // Headnode(s) can be and are determined inside BigDataPreconditionCheckUtil constructor. Involved nodes need to be given.
 *     'involved_nodes' : [ 813758235, 857878287, 2971971971 ],// uniqueKeys for node00[1-3]
 *
 *     'auto_start'     : true // invokes this.run() after constructing
 *   };
 *
 * hadoopPreconditionsChecker = new BigDataPreconditionCheckUtil(
 *   current,
 *   dock,
 *   outputObject,
 *   instructions
 * );
 *
 * </pre>
 * @param current - cluster object
 * @param dock - dock object (i.e., use getDock())
 * @param outputObject - instance of {@see MultipleCheckersProgressStepDisplayer}
 * @param checkInstructionsJson - json object containing configuration, i.e., actual checks.
 *
 * @constructor
 */
var BigDataPreconditionCheckUtil = function (current, dock, outputObject, checkInstructionsJson)
{
  // Private class data
  this.current                    = current;
  this.dock                       = dock;
  this.outputObject               = outputObject;
  this.checkInstructions          = checkInstructionsJson;

  this.getTarballs                = checkInstructionsJson.get_tarballs || false;
  this.checkTarballs              = checkInstructionsJson.check_tarballs || false;
  this.checkJavaHome              = checkInstructionsJson.check_javahome || false;
  this.involvedNodes              = checkInstructionsJson.involved_nodes || [];
  this.autoStart                  = !!checkInstructionsJson.auto_start;
  this.resultsArray               = [];

  this.javaRuntimeChecker         = false;
  this.tarballExistenceChecker    = false;

  this.checksInProgress           = 0;     // currently zero checks in progress
  this.checksFailed               = 0;     // currently zero checks failed
  this.checksSkippable            = false; // ui can set to true in order to skip the checks if they are not complete
  this.customStepMessages         = [];    // ui will display entries in this array in the stepdisplay

  if (this.getTarballs.length === 1) {
    this.getTarballsTarget        = this.getTarballs.shift();
  }
  if (this.checkTarballs.length >= 2) {
    this.checkTarballsTarget      = this.checkTarballs.shift();
    this.checkTarballsMode        = (this.checkTarballs.length >= 3) ? this.checkTarballs[1] : false;
    this.checkTarballs            = this.checkTarballs[0];
  }
  if (this.checkJavaHome.length === 2) {
    this.checkJavaHomeTarget      = this.checkJavaHome.shift();
    this.checkJavaHome            = this.checkJavaHome.shift();
  }

  // Get all master node keys
  this.masterNodeKeys = (function () {
                           var masterNodes = this.current.find('MasterNode');
                           if (masterNodes.length && masterNodes.length > 0) {
                             return masterNodes.map(function (node) { return node.get("uniqueKey"); });
                           }
                           return false;
                         }.bind(this))();
  // Filter it so only active master node remains
  this.dock.workerpool.getWorker(this.current.actualurl).call('cmpart', 'cmdaemonFailoverFullStatus', null, function (code, data) {
    if (code == 0 && data && data.length && data.length > 0) {
      this.preconditionschecker.masterNodesKeys = [data[0].masterKey];
    }
    // doesn't matter if this call fails, dwe'll just use the fallback default of checking both
  }, {'preconditionschecker': this});

  var getTargetsHelper = function (checkerTarget) {
                           switch (checkerTarget) {
                             case BigDataPreconditionCheckUtilTarget.HEADNODES_ONLY:
                               return this.masterNodeKeys;
                             case BigDataPreconditionCheckUtilTarget.INVOLVED_NODES:
                               return this.involvedNodes;
                           }
                         }.bind(this);

  /**
   * Helper function that formats an error line with nodes, and optionally the exact error.
   *
   * @param errorMessage - the error message
   * @param failedNodes - array of [node, failure message] pairs.
   * @param maxNumberNodes - max. number of node hostnames to concat to the error message.
   * @param showDetailedErrors - flag that also adds the exact underlying error encountered to the node hostname.
   */
  var formatErrorMessage = function(errorMessage, failedNodes, maxNumberNodes, showDetailedErrors)
  {
    // Make sure we won't splice the actual object
    var hostnames = JSON.parse(JSON.stringify(failedNodes)); // splice(0) didn't work for some reason

    // Convert uniqueKeys to hostnames
    hostnames = hostnames.splice(0, maxNumberNodes).map(function (i) {
      return current.find(i[0]).get('hostname') + (showDetailedErrors ? ': ' + i[1] : '');
    });

    // Comma separate the hostnames and append '...' in case there are nodes not included in the message
    hostnames = hostnames.join(', ') + (failedNodes.length - maxNumberNodes > 0 ? ', ...' : '');
    return errorMessage + ' (' + hostnames + ')';
  };

  /**
   * Properly destruct this object by calling stopAll(), will stop all timers etc.
   */
  this.stopAll = function () {
    if (this.getTarballsChecker) {
      this.getTarballsChecker.stop();
      this.getTarballsChecker = false;
    }
    if (this.javaRuntimeChecker) {
      this.javaRuntimeChecker.stop();
      this.javaRuntimeChecker = false;
    }
    if (this.tarballExistenceChecker) {
      this.tarballExistenceChecker.stop();
      this.tarballExistenceChecker = false;
    }
  };

  this.startGetTarballsCheck = function ()
  {
    this.checksInProgress += 1;

    this.getTarballsChecker = new NodesShellChecker(current, getTargetsHelper(this.getTarballsTarget));

    var callbackFunction = function(numNodesTotal,
                                    numNodesInBatch,
                                    numNodesSuccessful,
                                    numNodesCheckFailed,
                                    numNodesReturnInvalid,
                                    nodesCheckFailed,
                                    nodesReturnInvalid
    ){
      // this checker does not handle the updates for the outputObject progress displayer
      var numNodesChecked = (numNodesSuccessful + numNodesCheckFailed + numNodesReturnInvalid),
        maxNumberNodes  = 3, // maximum number of node hostnames to display as extra information
        nodesCheckFailedMessage,
        nodesReturnInvalidMessage,
        error = false;

      if (numNodesCheckFailed > 0)
        nodesCheckFailedMessage = formatErrorMessage('GetTarballs check failed.', nodesCheckFailed, maxNumberNodes);
      if (numNodesReturnInvalid > 0) {
        var showDetailedErrors = true;
        nodesReturnInvalidMessage = formatErrorMessage('GetTarballs not found on headnodes', nodesReturnInvalid, maxNumberNodes, showDetailedErrors);
      }

      this.self.outputObject.clearProgressLines('GetTarballs');

      // show progress
      if (!this.isCompleted)
        this.self.outputObject.addProgressLine('GetTarballs', 'STARTED', 'Checking tarballs on headnodes.');

      // show success
      if (numNodesSuccessful === numNodesTotal)
        this.self.outputObject.addProgressLine('GetTarballs', 'FINISHED', 'All required tarballs found on headnodes.');

      // show failure
      if (numNodesCheckFailed > 0) {
        this.self.outputObject.addProgressLine('GetTarballs', 'FAILED',   nodesCheckFailedMessage);
        error = true;
      }
      if (numNodesReturnInvalid > 0) {
        this.self.outputObject.addProgressLine('GetTarballs', 'FAILED',   nodesReturnInvalidMessage);
        error = true;
      }

      this.self.outputObject.display(this.isCompleted, this.self.results(), error);

      if (this.isCompleted) {
        this.self.checksInProgress -= 1;
        this.self.checksFailed += numNodesTotal - numNodesSuccessful;
      }
    };

    this.getTarballsChecker.setProgressCallback(callbackFunction.bind({'isCompleted' : false, 'self' : this}));
    this.getTarballsChecker.setCompletedCallback(callbackFunction.bind({'isCompleted' : true, 'self' : this}));

    // Simply check for existance of the tarballs by verifying they are all printed by /bin/ls [-1 <tarball> <tarball2> ... ]
    // If they don't exist, an error will be written to stderr which is not captured.
    this.getTarballsChecker.setCheckFunction(function (responseForNode) {
      var tarballsFound = responseForNode.trim().split("\n").sort();
      for (var i=0; i<tarballsFound.length; i++) {
        this.resultsArray.push(tarballsFound[i]);
      }
    }.bind(this));

    this.getTarballsChecker.start('ls', ['-1'].concat('/cm/local/apps/hadoop/'));
  };

  /**
   * Start Hadoop tarballs checking asynchrounously.
   *
   * Note that the Next button will be enabled, but will give a warning in case checking is still in progress..
   */
  this.startHadoopTarballsChecking = function ()
  {
    var isSummaryPageMode = this.checkTarballsMode === 'summaryPage';
    this.checksInProgress += 1;

    this.tarballExistenceChecker = new NodesShellChecker(current, getTargetsHelper(this.checkTarballsTarget));

    var dialog = new cvDialog(dock.myVersion + "error.xul",
                              'Not all tarballs could be found on the headnode. Please refer to the \'Hadoop Deployment Manual\' for the precise package that provides the tarballs. ' +
                              'Some are probably available from one of the following packages:\n' +
                              '- cm-apache-hadoop\n' +
                              '- cm-cloudera-hadoop\n' +
                              '- cm-hortonworks-hadoop\n'),
        dialogDisplayed = false;

    var callbackFunction = function(numNodesTotal,
                                    numNodesInBatch,
                                    numNodesSuccessful,
                                    numNodesCheckFailed,
                                    numNodesReturnInvalid,
                                    nodesCheckFailed,
                                    nodesReturnInvalid
    ){
      // this checker does not handle the updates for the outputObject progress displayer
      var numNodesChecked = (numNodesSuccessful + numNodesCheckFailed + numNodesReturnInvalid),
        maxNumberNodes  = 3, // maximum number of node hostnames to display as extra information
        nodesCheckFailedMessage,
        nodesReturnInvalidMessage,
        error = false;

      if (numNodesCheckFailed > 0)
        nodesCheckFailedMessage = formatErrorMessage('Tarballs check failed.', nodesCheckFailed, maxNumberNodes);
      if (numNodesReturnInvalid > 0) {
        var showDetailedErrors = true;
        nodesReturnInvalidMessage = formatErrorMessage('Tarballs not found on headnodes', nodesReturnInvalid, maxNumberNodes, showDetailedErrors);
      }

      this.self.outputObject.clearProgressLines('TarballVerifier');

      // show progress
      if (!this.isCompleted)
        this.self.outputObject.addProgressLine('TarballVerifier', 'STARTED', 'Checking tarballs on headnodes.');

      // show success
      if (numNodesSuccessful === numNodesTotal)
        this.self.outputObject.addProgressLine('TarballVerifier', 'FINISHED', 'All required tarballs found on headnodes.');

      // show failure
      if (numNodesCheckFailed > 0) {
        this.self.outputObject.addProgressLine('TarballVerifier', 'FAILED',   nodesCheckFailedMessage);
        error = true;
      }
      if (numNodesReturnInvalid > 0) {
        this.self.outputObject.addProgressLine('TarballVerifier', 'FAILED',   nodesReturnInvalidMessage);
        error = true;
      }

      this.self.outputObject.display(this.isCompleted, this.self.results());

      if (this.isCompleted) {
        this.self.checksInProgress -= 1;
        this.self.checksFailed += numNodesTotal - numNodesSuccessful;
      }

      if (error) {
        if (!dialogDisplayed && isSummaryPageMode) {
          dock.dialogs.showDialog(dialog);
          dialogDisplayed = true;
        }
      }
    };

    this.tarballExistenceChecker.setProgressCallback(callbackFunction.bind({'isCompleted' : false, 'self' : this}));
    this.tarballExistenceChecker.setCompletedCallback(callbackFunction.bind({'isCompleted' : true, 'self' : this}));

    // Simply check for existance of the tarballs by verifying they are all printed by /bin/ls [-1 <tarball> <tarball2> ... ]
    // If they don't exist, an error will be written to stderr which is not captured.
    this.tarballExistenceChecker.setCheckFunction(function (responseForNode) {
      var tarballsFound = responseForNode.trim().split("\n"),
      tarballsNotFound = this.checkTarballs.filter(function (tarball) {
        for (var i=0; i<tarballsFound.length; i++) {
          if (tarballsFound[i].indexOf(tarball +':') === 0 &&
              tarballsFound[i].indexOf('No such file or directory') < 0) {
            return false;
          }
        }
        return true;
      });

      for (var i=0; i<this.checkTarballs.length; i++) {
        var tarball = this.checkTarballs[i];
        this.resultsArray.push([tarball, tarballsNotFound.indexOf(tarball) === -1 ? true : false]);
      }

      if (tarballsNotFound.length > 0) {
        return 'not found: ' + tarballsNotFound.join(', ');
      }
    }.bind(this));

    this.tarballExistenceChecker.start('file', this.checkTarballs);
  };

  /**
   * Start Java/JRE Home checking asynchrounously.
   *
   * Note that the Next button will be enabled, but will give a warning in case checking is still in progress..
   */
  this.startJavaRuntimeChecking = function ()
  {
    this.checksInProgress += 1;

    var dialog = new cvDialog(dock.myVersion + "error.xul", 'The Java executable file \'bin/java\' could not be found in specified JRE folder  \'' + this.checkJavaHome + '\' on all nodes that are part of this Hadoop instance.'),
        dialogDisplayed = false;

    var batchSize      = 100;
    var batchTimeout   = 30; // seconds
    this.javaRuntimeChecker = new NodesShellChecker(current, getTargetsHelper(this.checkJavaHomeTarget), batchSize, batchTimeout);

    var callbackFunction = function(numNodesTotal,
                                    numNodesInBatch,
                                    numNodesSuccessful,
                                    numNodesCheckFailed,
                                    numNodesReturnInvalid,
                                    nodesCheckFailed,
                                    nodesReturnInvalid
    ){
      var numNodesChecked   = (numNodesSuccessful + numNodesCheckFailed + numNodesReturnInvalid),
        maxNumberNodes      = 3, // maximum number of node hostnames to display as extra information
        nodesCheckFailedMessage,
        nodesReturnInvalidMessage,
        error = false;

      if (numNodesCheckFailed > 0)
        nodesCheckFailedMessage = formatErrorMessage(numNodesCheckFailed, nodesCheckFailed, maxNumberNodes);
      if (numNodesReturnInvalid > 0)
        nodesReturnInvalidMessage = formatErrorMessage(numNodesReturnInvalid, nodesReturnInvalid, maxNumberNodes);

      this.self.outputObject.clearProgressLines('JavaHomeVerifier');

      // show progress
      if (!this.isCompleted)
        this.self.outputObject.addProgressLine('JavaHomeVerifier', 'STARTED', 'Number of nodes checked for JRE: ' + numNodesChecked + ' / ' + numNodesTotal);

      // show success
      if (numNodesSuccessful === numNodesTotal)
        this.self.outputObject.addProgressLine('JavaHomeVerifier', 'FINISHED', 'Java Runtime Environment (JRE) found on all ' + numNodesTotal + ' nodes.');
      else if (numNodesSuccessful > 0)
        this.self.outputObject.addProgressLine('JavaHomeVerifier', 'FINISHED', 'Number of nodes on which JRE home does not contain \'bin/java\': ' + numNodesSuccessful);

      // show failure
      if (numNodesCheckFailed > 0) {
        this.self.outputObject.addProgressLine('JavaHomeVerifier', 'FAILED',   'Number of nodes on which the check itself failed: ' + nodesCheckFailedMessage);
        error = true;
      }
      if (numNodesReturnInvalid > 0) {
        this.self.outputObject.addProgressLine('JavaHomeVerifier', 'FAILED',   'Number of nodes on which JRE home does not contain \'bin/java\': ' + nodesReturnInvalidMessage);
        error = true;
      }

      this.self.outputObject.display(this.isCompleted, this.self.results());

      if (this.isCompleted) {
        this.self.checksInProgress -= 1;
        this.self.checksFailed += numNodesTotal - numNodesSuccessful;
      }

      if (error) {
        if (!dialogDisplayed) {
          dock.dialogs.showDialog(dialog);
          dialogDisplayed = true;
        }
      }
    };

    this.javaRuntimeChecker.setProgressCallback(callbackFunction.bind({'isCompleted' : false, 'self' : this}));
    this.javaRuntimeChecker.setCompletedCallback(callbackFunction.bind({'isCompleted' : true, 'self' : this}));

    // Simply check for existance of the java executable, if it exists ls -1 will echo it.
    // If it doesn't exist, an error will be written to stderr which is not captured and the output will be an empty string.
    this.javaRuntimeChecker.setCheckFunction(function (responseForNode) {
      if (responseForNode.trim().length === 0) {
        return 'Java/JRE directory containing bin/java does not exist';
      }
    });
    this.javaRuntimeChecker.start('ls', ['-1', this.checkJavaHome + '/bin/java' ]);
  };

  /**
   * Run the defined checks or display error in UI
   */
  this.run = function () {
    // Start check: tarballs exist on Head nodes?
    if (!this.masterNodeKeys) {
      this.customStepMessages.push(['FAILED', 'Could not initialize check to verify Hadoop tarballs on (active) Head node']);
      return;
    }
    if (this.getTarballs) {
      this.startGetTarballsCheck();
    }
    if (this.checkTarballs) {
      this.startHadoopTarballsChecking();
    }
    if (this.checkJavaHome) {
      this.startJavaRuntimeChecking();
    }
  };

  /**
   * Helper function to prompt the user before proceeding setup in case something was not valid.
   *
   * @param skipFunctionCallback
   * @returns {boolean}
   */
  this.checkPrerequisites = function (skipFunctionCallback) {
    if (this.checksSkippable || (this.checksInProgress === 0 && this.checksFailed === 0)) {
      return true; // good to go
    }
    var args = new Object();
    if (this.checksInProgress !== 0) {
      args['question'] = "Are you sure you want to continue even though the checks have not yet completed?";
    }
    else if (this.checksFailed !== 0) {
      args['question'] = "Are you sure you want to continue even though not all checks have succeeded?";
    }
    var d = new cvDialog(dock.myVersion + "question.xul", args);
    d.onOk = function() {
      this.checksSkippable = true;
      skipFunctionCallback();
    }.bind(this);
    this.dock.dialogs.showDialog(d);
    return false;
  }

  this.results = function () {
    return this.resultsArray;
  }

  if (this.autoStart) this.run();

};

/**
 * Use this on change handler on a checkbox
 * @param elem
 */
function onChangeDefaultJavaHomeCheckbox(elem, userChoices)
{
  var defaultJavaHomeElem = document.getElementById('defaultJavaHome'),
    customJavaHomeElem    = document.getElementById('customJavaHome') || document.getElementById('javaHome'),
    defaultJavaCheckbox   = document.getElementById('useDefaultJavaHome');

  defaultJavaHomeElem.style.display = ( elem.checked) ? '' : 'none';
  customJavaHomeElem.style.display  = (!elem.checked) ? '' : 'none';

  userChoices.useDefaultJavaHome = elem.checked;
};

function setJavaHomeToUse(userChoices) {
  var defaultJavaHomeElem = document.getElementById('defaultJavaHome'),
      customJavaHomeElem  = document.getElementById('customJavaHome'),
      defaultJavaCheckbox = document.getElementById('useDefaultJavaHome');

  userChoices.javaHome = defaultJavaCheckbox.checked ? defaultJavaHomeElem.value : customJavaHomeElem.value;
}

function setJavaHomeBasedOnOS(currentCluster, destObject, destFormElement, java180) {
  if (java180 == undefined) {
    java180 = false;
  }
  
  var sic = new dock.cvSysInfoCollector(),
    masterNodes = currentCluster.find('MasterNode'),
    masterNodeKey,
    javaHomeInputElement = document.getElementById('defaultJavaHome') || document.getElementById('javaHome'),
    throbberImageElement = document.getElementById('javaHomeThrobber'),
    nextButton = document.getElementById('next');

  // Helper function: Enable/disable read-only property on the java home input, Next button and visibility of throbber image
  function makeRelatedUiElementsReadOnly(lock) {
    if (lock) {
      nextButton.setAttribute('disabled', true);
    } else {
      nextButton.removeAttribute('disabled');
    }
    throbberImageElement.style.display = lock ? 'block' : 'none';
  }

  if (masterNodes.length && masterNodes.length > 0) {
    masterNodeKey = masterNodes[0].get("uniqueKey");

    // Also create a restore timer just in case something in the creation of the worker creates an exception
    //  unfortunately we cannot use something like a destructor for this in javascript.
    var restoreIfHangsForMoreThanFiveSecondsTimer = setTimeout(makeRelatedUiElementsReadOnly.bind(this, false), 5000);

    makeRelatedUiElementsReadOnly(true);

    dock.workerpool.getWorker(current.actualurl).call('cmdevice', 'getSysInfoCollector', [ masterNodeKey ], function(code, data) {
      makeRelatedUiElementsReadOnly(false);

      // Also if the fallback timer is still running, kill it.
      clearTimeout(restoreIfHangsForMoreThanFiveSecondsTimer);

      // don't bother the cmgui user with a possible error, we also have the placeholder as a hint
      if (code != 0) {
        return;
      }

      // Update in destination object
      if (data.osFlavor) {
        if (java180) {
          destObject.javaHome = getDefaultJava180HomeByOSFlavor(data.osFlavor);
        } else {
          destObject.javaHome = getDefaultJavaHomeByOSFlavor(data.osFlavor);
        }
      }

      destFormElement.value           = destObject.javaHome;

    }, current);
  }
}

function getDefaultJavaHomeByOSFlavor(osFlavor) {
  var isRHEL7     = osFlavor.indexOf('RedHat7')   === 0,
    isCENTOS7   = osFlavor.indexOf('CentOS7')   === 0,
    isSLES11    = osFlavor.indexOf('SLES11')    === 0,
    isSLES12    = osFlavor.indexOf('SLES12')    === 0;

  if (isRHEL7 || isCENTOS7)
    return '/usr/lib/jvm/jre-1.7.0-openjdk/';
  if (isSLES11 || isSLES12) {
    return '/usr/java/jre1.7.0_80/';
  }
  // default for rhel6
  return '/usr/lib/jvm/jre-1.7.0-openjdk.x86_64/';
}

function getDefaultJava180HomeByOSFlavor(osFlavor) {
  var isRHEL7   = osFlavor.indexOf('RedHat7')   === 0,
    isCENTOS7   = osFlavor.indexOf('CentOS7')   === 0,
    isSLES11    = osFlavor.indexOf('SLES11')    === 0,
    isSLES12    = osFlavor.indexOf('SLES12')    === 0;

  if (isRHEL7 || isCENTOS7) {
    return '/usr/lib/jvm/jre-1.8.0-openjdk/';
  } else if (isSLES11 || isSLES12) {
    return '/usr/java/jre1.8.0_112/';
  }

  // default for rhel6
  return '/usr/lib/jvm/jre-1.8.0-openjdk.x86_64/';
}

function validateJavaHome(defaultJavaHomeCheckboxId, defaultJavaHomeId, customJavaHomeId, userChoices) {
  var checkbox = document.getElementById(defaultJavaHomeCheckboxId),
      defaultJavaHome = document.getElementById(defaultJavaHomeId),
      customJavaHome = document.getElementById(customJavaHomeId);

  if (checkbox.checked) {
    if (defaultJavaHome.value.length === 0) {
      checkbox.checked = false;
      onChangeDefaultJavaHomeCheckbox(checkbox, userChoices);
      return 'Java Runtime Environment home could not be determined, please specify it manually';
    }
  } else {
    if (customJavaHome.value.length === 0) {
      return 'Java Runtime Environment home was not specified';
    }
  }
  return true;
}
