/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

function determineNodesApplied(current) {
  var nodesapplied = new Array();
  if (!current) return nodesapplied;

  var nodes = current.owner.owner.get('nodes');
  for(var k = 0; k < nodes.length; k++) {
    var key = nodes[k] + "/" + current.owner.get('childType');
    var pr = 0;
    var ind = 0;
    for(var m = 0; m < map[key].length; m++) {
      if(map[key][m].priority > pr) {
        pr = map[key][m].priority;
        ind = m;
      }
    }
    if(current.owner.owner.resolveName() == map[key][ind].name) {
      nodesapplied.push(nodes[k]);
    }
  }
  return nodesapplied;
}

/**
 * Check a condition with a shell command on given array of nodes for a cluster using configurable batch sizes.
 *
 * This class provides a few callbacks so you can keep a UI up to date for example.
 *
 * It should be possible in the future to extend this class so it can be used by the parallel cluster shell code.
 * For now it's used by the Hadoop Wizard to perform certain checks like whether JRE directory is correct
 *  or tarballs for Hadoop exist on the Head node.
 */
var NodesShellChecker = function (current, nodes, batchSize, batchTimeout) {

  batchSize    = batchSize || 10;    // default batch size
  batchTimeout = batchTimeout || 30; // timeout in seconds (checks will fail after this value)


  return {
    'current'                  : current, 
    'numNodesTotal'            : nodes.length,
    'nodesQueued'              : nodes,
    'nodesProcessing'          : [],
    'nodesSuccessful'          : [],
    'nodesCheckFailed'         : [],
    'nodesReturnInvalid'       : [],
    'trackers'                 : [],
    'command'                  : '/bin/uname',
    'args'                     : [ '-a' ],
    'progressCallbackFunction' : function () {},
    'completeCallbackFunction' : function () {},
    'checkFunction'            : function (stdout) { if (stdout.trim().length === 0) return 'output is empty'; },
    'checkTimer'               : false, // reference to the periodic check timer

    // public methods
    'setProgressCallback' : function (callbackFunction) {
      this.progressCallbackFunction = callbackFunction;
    },

    'setCompletedCallback' : function (callbackFunction) {
      this.completeCallbackFunction = callbackFunction;
    },

    'setCheckFunction' : function (checkFunction) {
      this.checkFunction = checkFunction;
    },

    /**
     * start a command with given arguments
     *
     * Example usage: start('/bin/ls', ['/etc/passwd', '/etc/resolv.conf'])
     * Example usage: start('/bin/sh', ['-c', 'ls /etc/passwd /etc/resolv.conf'])
     *
     * A shorthand for the latter syntax is possible by not starting the command parameter with a forward slash (/)
     * Therefore the following example will be the same.
     *     start('ls', ['/etc/passwd', '/etc/resolv.conf']) will become ...
     * ... start('/bin/sh', ['-c', 'ls /etc/passwd /etc/resolv.conf'])
     *
     * @param command
     * @param args
     */
    'start' : function (command, args) {
      if (!command.startsWith('/')) {
        args = ['-c', command + ' ' + args.join(' ')];
        command = '/bin/sh';
      }
      this.command = command;
      this.args = args;
      this.processBatch();
    },

    'stop' : function () {
      if (this.checkTimer) {
        clearTimeout(this.checkTimer);
        this.checkTimer = false;
      }
    },

    // implementation
    'processBatch' : function () {
      if (this.nodesQueued.length == 0) {
        this.completeCallbackFunction(
          this.numNodesTotal,
          this.numNodesTotal - this.nodesQueued.length,
          this.nodesSuccessful.length,
          this.nodesCheckFailed.length,
          this.nodesReturnInvalid.length,
          this.nodesCheckFailed,
          this.nodesReturnInvalid
        );
        return;
      }

      // Slice nodes for processing from the queued nodes array
      this.nodesProcessing = this.nodesQueued.splice(0, batchSize);
      
      // Create ProgramRunnerInput instance and execute it via API call
      this.callProgramRunner(this.createProgramRunnerInput());

      // This will poll result tracker returns from the API call
      // If all responses for are processed it will call this function again
      this.checkTrackerResults();
    },

    'checkTrackerResults' : function () {
      for (var i=0; i<this.trackers.length; i++) {
        this.checkTrackerResult(this.trackers[i]);
      }

      var numNodesBatchProcessed = this.numNodesTotal - this.nodesQueued.length; 

      var resultsAvailableForAllNodes = numNodesBatchProcessed === (this.nodesSuccessful.length +
                                                                    this.nodesCheckFailed.length +
                                                                    this.nodesReturnInvalid.length);
      this.progressCallbackFunction(
          this.numNodesTotal,
          this.nodesProcessing.length,
          this.nodesSuccessful.length,
          this.nodesCheckFailed.length,
          this.nodesReturnInvalid.length,
          this.nodesCheckFailed,
          this.nodesReturnInvalid
      );

      if (resultsAvailableForAllNodes)
        this.processBatch(); // next batch
      else
        this.checkTimer = setTimeout(this.checkTrackerResults.bind(this), 1000); // keep checking
    },

    'checkTrackerResult' : function (tracker) {
      var trackerResult, nodeId, responseForNode, i, error;

      trackerResult = dock.getAlreadyArrivedShellTracker(tracker.tracker);

      for (i=0; i<trackerResult.length; i++) {
        nodeId          = trackerResult[i][1];
        responseForNode = trackerResult[i][2];

        error = this.checkFunction(responseForNode);
        if (error === '' || typeof error !== 'string') { // no error
          this.nodesSuccessful.push(nodeId);
          tracker.remainingNodes = tracker.remainingNodes.filter(function (i) { return i !== nodeId; }); // remove this nodeId from remaining list

        } else {
          this.nodesReturnInvalid.push([nodeId, error]);
          tracker.remainingNodes = []
        }
      }

      // mark nodes as check failed if they take too long to acknowledge
      if ((((new Date()) - tracker.trackerStarted) / 1000) > batchTimeout) {
        this.nodesCheckFailed = this.nodesCheckFailed.concat(tracker.remainingNodes.map(
          function (i) { return [i, 'timeout']; }
        ));
      }
    },

    'createProgramRunnerInput' : function () {
      var programRunnerInput = new dock.cvProgramRunnerInput();
      programRunnerInput.cmd  = this.command;
      programRunnerInput.args = this.args;
      // Uncomment the following two lines to simulate random delays (between 0-10 seconds per node)
      // !!Note that it's not command injection safe to do this
      //programRunnerInput.cmd = '/bin/sh';
      //programRunnerInput.args = ['-c', 'sleep ${RANDOM:0:1} && ' + this.command + ' ' + this.args.join(' ')];
      programRunnerInput.user = this.current.syslogin;
      programRunnerInput.mergeCoutCerr = false;
      return programRunnerInput;
    },

    'callProgramRunner' : function (programRunnerInput) {
      dock.workerpool.getWorker(this.current.actualurl).call(
        'cmproc',
        'runCommand',
        [ this.nodesProcessing, programRunnerInput, this.current.sessionid ],
        function(code, data) {
          if (code == 0) {
            // Create a tracker we can regularly poll for completion
            this.trackers.push({'trackerStarted' : new Date(),
                                'tracker'        : data.input.tracker,
                                'remainingNodes' : this.nodesProcessing
                               });
          } else {
            // Unfortunately mark all nodes for this batch as in being in error due to the api call failure
            this.nodesCheckFailed = this.nodesCheckFailed.concat(this.nodesProcessing.map(
              function (i) { return [i, 'JSON call cmproc:runCommand (' + code + ')']; }
            ));
            println('JSON call cmproc:runCommand (' + code + ')');
            printall(data);
          }
        }.bind(this),
        this.current
      );
    }
  };
};

/**
 * Mask to instruct overwriting of the decommission map mask in CMDaemon.
 * 
 * In binary the mask is:              Which will be inverted by CMDaemon at some point resulting in:
 *  01111111111111111111111111111111    10000000000000000000000000000000
 *                                      ||`-> substract bit is "off"
 *                                      |`--> merge bit is "off"
 *                                      `---> (non-zero value for this number so it's not an undrain.)
 * I.e., this mask can be used as a base to enable decommissioning of HDFS:
 *  10000000000000000000000000000001
 *                               ||`-> decommission HDFS is "on"
 *                               |`--> decommission Mapreduce is "off"
 *                               `---> decommission Yarn is "off"
 * If i.e. Mapreduce was "on" before this drain command it will be "overwritten" 
 *  with this "Overwrite" mask.
 */
var HADOOP_DRAIN_OVERWRITE = 0x7fffffff;
/**
 * Mask to instruct merging of the decommission map mask in CMDaemon.
 * 
 * @see HADOOP_DRAIN_OVERWRITE
 *
 * The only difference is that an existing mask will be combined, i.e.:
 *  11000000000000000000000000000010 (existing mask in CMDaemon)
 *  11000000000000000000000000000001 (the example mask)
 *  -------------------------------- +
 *  11000000000000000000000000000011 (resulting new mask)
 */
var HADOOP_DRAIN_MERGE     = 0x3fffffff;
/**
 * Mask to instruct substracting of the decommission map mask in CMDaemon.
 * 
 * @see HADOOP_DRAIN_OVERWRITE
 *
 * The only difference is that an existing mask will be substracted, i.e.:
 *  11100000000000000000000000000011 (existing mask in CMDaemon)
 *  11100000000000000000000000000001 (the example mask)
 *  -------------------------------- -
 *  11100000000000000000000000000010 (resulting new mask)
 */
var HADOOP_DRAIN_SUBSTRACT = 0x1fffffff;

/**
 * Generate a decommission mask with specific exclude bits set for: hdfs, mapreduce, yarn.
 *
 * Example usage:
 *  getHadoopDrainAction(['decommissionHdfs', 'decommissionYarn'])  -> 2147483643
 *                                                                  -> 01111111111111111111111111111011
 *
 * this will create a mask that will trigger a decommission only for HDFS and Yarn.
 * Currently supported values are:
 * - decommissionHdfs
 * - decommissionMapreduce
 * - decommissionYarn
 */
function getHadoopDrainAction(decommission, mode) {
  var modeMask = !mode ? HADOOP_DRAIN_OVERWRITE : mode;

  var obj = {
    'mask'                  : modeMask,
    'decommissionHdfs'      : function (value) { obj.set(0, value); },
    'decommissionMapreduce' : function (value) { obj.set(1, value); },
    'decommissionYarn'      : function (value) { obj.set(2, value); },
    'set'                   : function (bit, value) {
                                if (value)
                                  this.mask = this.mask | 1<<bit;
                                else
                                  this.mask = this.mask & ~(1<<bit);
                              }
  };
  // In cmdaemon the implementation considers some backwards compatibility values,
  // zero, one and > one are already special cases:
  //  * 0 = undrain
  //  * 1 = decommission all
  //  * > 1 = is *decremented* and applied as a file mask
  // we simply set the last bit to one, so it won't clash with any of the above special cases.
  // i.e.: when we exclude the first three files (dfs, mapreduce and yarn),
  //       this will create the mask 01111111111111111111111111111000 to be read by HadoopFileMask
  //       the mask will be decremented because its the special case "> 1",
  //       therefore it becomes 10000000000000000000000000000111
  //                            ||`-> subtract (0=No, 1=Yes) ||`-> exclude dfs
  //                            |`-> merge (0=No, 1=Yes)     |`--> exclude mapreduce
  //                            `-> ignored anyway           `---> exclude yarn
  // @see HadoopJobScheduler::drainNodes() - the special cases are converted
  // @see HadoopConfigurator::performUpdate() - the file masks are interpreted
  // obj.set(32 - 1, false); // already in the mask
  for (var i=0; i < (decommission ? decommission.length : 0); i++) {
    if (obj[decommission[i]]) obj[decommission[i]]();
  }

  // Part of the "> 1" special case is that the drainValue is *decremented* first to keep
  //  backwards compatibility, so we have to increment by one..
  obj.mask = obj.mask + 1;
  return obj.mask;
};

/**
 * Parameter value for cmhadoop::getHadoopDecommissionStatusResponse's includeJMX flag.
 *
 * Including JMX requires CMDaemon to query Hadoop services.
 * This extra information is not always necessary, for example if you are only interested
 *  in the Intention of the node(s), and not the actual Status according to Hadoop.
 */
var HADOOP_DECOMMISSION_STATUS_INCLUDE_JMX = true;
/**
 * Parameter value for cmhadoop::getHadoopDecommissionStatusResponse's includeJMX flag.
 *
 * @see HADOOP_DECOMMISSION_STATUS_INCLUDE_JMX
 */
var HADOOP_DECOMMISSION_STATUS_EXCLUDE_JMX = false;

/**
 * Generic handler function that can be used to plug in cmhadoop::getHadoopDecommissionStatus calls.
 *
 * Example usage:
 * dock.workerpool.getWorker(current.cluster.actualurl).call(
 *   'cmhadoop',
 *   'getHadoopDecommissionStatus',
 *   [hdfskey, 'hdfs', false],
 *   handle_getHadoopDecommissionStatusResponse.bind({'refreshFunc' : refreshFunc, 'dock' : dock})
 * );
 *
 * This is in hadoopcommon because this call is used in hadoop{hdfs|mapred|yarn}actions.js.
 */
function handle_getHadoopDecommissionStatusResponse(code, data)
{
  var nodeintentions = undefined;
  var JMX_stati      = undefined;
  if(code == 0) {
    this['dock'].unwaitIcon();
    if (data.intention && data.intention.length) {
      nodeintentions = {};
      JMX_stati = {};
      for (var i=0; i<data.nodes.length; i++) {
        if (data.intention && data.intention[i])
          nodeintentions[data.uniqueKeys[i]] = data.intention[i];
        if (data.status && data.status[i])
          JMX_stati[data.uniqueKeys[i]] = data.status[i];
      }
    }
  } else {
    this['dock'].jsonCallError(this, code, data);
    this['dock'].unwaitIcon();
  }
  this['refreshFunc'](nodeintentions, JMX_stati);
}

/**
 * Handle cmjob::drainNodes XHR response by opening a result dialog with the results.
 * 
 * Instructs this dialog to load cmhadoop::getHadoopDecommissionStatus data in the grid.
 */
function handle_drainNodesResponse(code, data) {
  var self    = this['self'];
  var hdfskey = this['hdfskey'];
  var filter  = this['filter'];

  if (dock) dock.unwaitIcon();

  if(code == 0) {
    showDecommissionStatusOverviewWindow(self, hdfskey, filter);
  } else {
    dock.jsonCallError(self, code, data);
  }
};

function showDecommissionStatusOverviewWindow(self, hdfskey, filter)
{
  dock.waitIcon();
  var args = new Object();
  args['initialize'] = function (handlers) {
    dock.workerpool.getWorker(current.cluster.actualurl).call('cmhadoop', 'getHadoopDecommissionStatus', [hdfskey, filter, HADOOP_DECOMMISSION_STATUS_INCLUDE_JMX], 
      function(code, data) {
        if(code == 0) {
          handlers.update(data);
        } else {
          dock.jsonCallError(self, code, data);
          handlers.error();
        }
      }
    );
  };
  args['cluster'] = self.current.cluster;
  var d = new dock.cvDialog(dock.myVersion + "hadoop/decommissionstatus.xul", args);
  dock.dialogs.showDialog(d);
  dock.unwaitIcon();
}

function isCorrectlyConfigured(cluster, hdfsKey) {
  var allOverlays = cluster.find('ConfigurationOverlay');
  for (var i = 0; i < allOverlays.length; i++) {
    var roles = allOverlays[i].getHadoopRoles(hdfsKey);
    if (roles.length > 0) {
      return true;
    }
  }
  return false;
}

/**
 * Get all configuration overlays for given HDFS instance.
 * 
 * @param current - HadoopHDFS instance
 * @returns overlay[]
 */
function getAllOverlays(current) {
  return current.cluster.find('ConfigurationOverlay');
}

/**
 * Get all Hadoop nodes involved in given HDFS instance.
 * 
 * It will construct the list of nodes by consulting the configuration overlays
 *  related to the instance.
 *  
 * @param current - HadoopHDFS instance
 * @returns int[] - Array of all the Nodes uniqueKey
 */
// @param current  - HadoopHDFS instance.
function getAllHadoopNodeKeys(current) {
  var allHadoopNodeKeys = new Array();
  var HadoopNodeKeysObj = new Object();

  var hdfsKey = current.get('uniqueKey');
  var overlays = getAllOverlays(current);
  for(var k = 0; k < overlays.length; k++) {
    // get nodes and Hadoop-related(!) roles for configuration overlay
    var nodes = overlays[k].get('nodes');
    var roles = overlays[k].getHadoopRoles(hdfsKey, false);

    if (roles.length > 0) {
      for (var i = 0; i < nodes.length; i++) {
        HadoopNodeKeysObj[nodes[i]] = 1;
      }
    }
  }
  allHadoopNodeKeys = Object.keys(HadoopNodeKeysObj);
  return Object.keys(HadoopNodeKeysObj);
}
