/*
 * Copyright (c) 2004-2015 Bright Computing Holding BV. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of
 * Bright Computing Holding BV ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Bright Computing Holding BV or its subsidiaries.
 */

/*
 * AttributeListable Classes
 */
function AttributeListable(currentObject, attribute) {
  this.currentObject = currentObject;
  this.attribute = attribute;
  this.inputValue = null;
  this.outputValue = null;

  this.init = function(currentObject, attribute) {
    this.currentObject = currentObject;
    this.attribute = attribute;
  };

  this.initInputValue = function() {
    this.inputValue = this.currentObject.get(this.attribute);
    try {
      this.inputToOutput();
    } catch(e) {
      println('Input attribute ' + this.attribute + ' is not compatible with this class.');
    }
  };

  this.get = function() {
    return this.outputValue;
  };

  this.change = function(row, col, value) {
    this.outputValue[row][col] = value;
    this.changeCallback();
  };

  this.add = function(newItem) {
    this.outputValue.push(newItem);
    this.changeCallback();
  };

  this.del = function(index) {
    this.outputValue.splice(index, 1);
    this.changeCallback();
  };

  this.save = function() {
    this.currentObject.set(this.attribute, this.outputToInput());
    this.initInputValue();
    this.saveCallback();
  };

  this.saveCallback = function() {
    return true;
  };

  this.changeCallback = function() {
    return true;
  }

  this.inputToOutput = function() {
    // Needs to be implemented by the extender
  };

  this.outputToInput = function() {
    // Needs to be implemented by the extender
  };
};


function JsonAttributeListable(currentObject, attribute) {

  this.inputFilter = function() {
    return true;
  };

  this.setInputFilter = function(filter) {
    this.inputFilter = filter;
  };

  this.outputFilter = function(value) {
    return value;
  };

  this.setOutputFilter = function(filter) {
    this.outputFilter = filter;
  };

  this.inputToOutput = function() {
    var o = JSON.parse(this.inputValue);
    this.outputValue = Object.keys(o).filter(this.inputFilter
    ).map(function(k) {
      return [k, o[k]];
    });
    return this.outputValue;
  };

  this.outputToInput = function() {
    var o = {};
    for(var i = 0; i < this.outputValue.length; i++) {
      o[this.outputValue[i][0]] = this.outputValue[i][1];
    }
    o = this.outputFilter(o);
    this.inputValue =  JSON.stringify(o);
    return this.inputValue;
  };

  this.init(currentObject, attribute);
  this.initInputValue();
};

JsonAttributeListable.prototype = new AttributeListable();
JsonAttributeListable.prototype.constructor = JsonAttributeListable;


function ObjectAttributeListable(currentObject, attribute) {

  this.inputToOutput = function() {
    var o = this.inputValue;
    this.outputValue = Object.keys(o).map(function(k) { return [k, o[k]]; });
    return this.outputValue;
  };

  this.outputToInput = function() {
    var o = {};
    for(var i = 0; i < this.outputValue.length; i++) {
      o[this.outputValue[i][0]] = this.outputValue[i][1];
    }
    this.inputValue =  o;
    return this.inputValue;
  };

  this.init(currentObject, attribute);
  this.initInputValue();
};
ObjectAttributeListable.prototype = new AttributeListable();
ObjectAttributeListable.prototype.constructor = ObjectAttributeListable;


function SingleArrayAttributeListable(currentObject, attribute) {

  this.inputToOutput = function() {
    var o = this.inputValue;
    this.outputValue = Object.keys(o).map(function(k) { return [o[k]]; });
    return this.outputValue;
  };

  this.outputToInput = function() {
    var o = [];
    for(var i = 0; i < this.outputValue.length; i++) {
      o.push(this.outputValue[i][0]);
    }
    this.inputValue =  o;
    return this.inputValue;
  };

  this.init(currentObject, attribute);
  this.initInputValue();
};
SingleArrayAttributeListable.prototype = new AttributeListable();
SingleArrayAttributeListable.prototype.constructor = SingleArrayAttributeListable;


function ArrayObjectAttributeListable(currentObject, attribute) {
  this.object = null;
  this.columnOrder = {};
  this.columnFilter = [];
  this.columnHeaders = [];
  this.rowOrder = [];
  this.originalValuesForRows = [];

  this.setObject = function(object) {
    this.object = object;
  };

  this.setColumnOrder = function(columnOrder) {
    this.columnOrder = columnOrder;
    this.initColumnFilter();
  };

  this.initColumnFilter = function() {
    this.columnFilter = Object.keys(this.columnOrder);
  };

  // Use this function to overrule that default for columns.
  // (by default, ucfirst() on the columnFilter names will be used)
  this.setColumnHeaderLabels = function(headers) {
    this.columnHeaders = headers;
  };

  this.addRowOrder = function(rowOrder) {
    this.rowOrder.push(rowOrder);
  };

  this.inputToOutput = function() {
    var o = this.inputValue;
    var order = this.columnOrder;
    this.outputValue = Object.keys(o).map(function(k1) {
      return Object.keys(o[k1]).filter(function(e){
        return (order[e]!==undefined);
      }).sort(function(a, b){
        return (order[a]>=order[b]);
      }).map(function(k2) {
        return o[k1][k2];
      });
    });
    for(var i = 0; i < this.rowOrder.length; i++) {
      this.outputValue  = new this.rowOrder[i](this.outputValue).get();
    }
    return this.outputValue;
  };

  this.rememberInitialValues = function() {
    this.initInputValue();
    for (var i = 0; i < this.outputValue.length; i++) {
      this.originalValuesForRows.push(JSON.stringify(this.outputValue[i]));
    }
  }

  this.hasChangedSinceInitialization = function (index) {
    return JSON.stringify(this.outputValue[index]) != this.originalValuesForRows[index];
  }

  this.outputToInput = function() {
    var a = [];
    for(var i = 0; i < this.outputValue.length; i++) {
      var o = new this.object();
      for(var j = 0; j < this.outputValue[i].length; j++) {
        o[this.columnFilter[j]] = this.outputValue[i][j];
        o['modified'] = this.hasChangedSinceInitialization(i);
      }
      a.push(o);
    }
    this.inputValue =  a;
    return this.inputValue;
  };

  this.init(currentObject, attribute);

  // It is safe to remember initial values only here, for succint edits a new AttributeListable object
  // will be created always.
  this.rememberInitialValues();
};
ArrayObjectAttributeListable.prototype = new AttributeListable();
ArrayObjectAttributeListable.prototype.constructor = ArrayObjectAttributeListable;


function ObjectDualViewAttributeListable(inputValueAll, currentObject, attribute) {
  this.inputValueSelection = inputValueAll;
  this.outputValueSelection = null;

  this.columnOrder = {};

  this.setColumnOrder = function(columnOrder) {
    this.columnOrder = columnOrder;
  };

  this.inputToOutput = function() {
    this.inputToOutputFiltered(false);
    return this.inputToOutputFiltered(true);
  };

  this.getSelection = function() {
    return this.outputValueSelection;
  };

  this.getSelected = function() {
    return this.get();
  };

  this.inputToOutputFiltered = function(inverted) {
    var order = this.columnOrder;
    var filter = this.inputValue;
    var filterInvert = inverted;
    var o = this.inputValueSelection;
    var outputValue = Object.keys(o)
    .filter(function(k){
      if (filterInvert) {
        return filter.indexOf(o[k].object.uniqueKey) >= 0;
      } else {
        return filter.indexOf(o[k].object.uniqueKey) < 0;
      }
    })
    .map(function(k) {
      var result = [];
      Object.keys(order).forEach(function(key) {
        result.push(o[k].object[key]);
      });
      return result;
    });
    if (filterInvert) {
      this.outputValue = outputValue;
    } else {
      this.outputValueSelection = outputValue;
    }
    return this.outputValue;
  };

  this.outputToInput = function() {
    throw 'ObjectListable.outputToInput should not be called.';
  };

  this.add = function(indexes) {
    var filter = this.inputValue;
    var o = this.inputValueSelection;
    this.inputValue = this.inputValue.concat(Object.keys(o)
    .filter(function(k) {
      return filter.indexOf(o[k].object.uniqueKey) < 0;
    })
    .map(function(k) {
      return o[k].object.uniqueKey;
    })
    .filter(function(element, index, array){
      return indexes.indexOf(index) >= 0;
    }));
  };

  this.del = function(indexes) {
    var filter = this.inputValue;
    var o = this.inputValueSelection;
    this.inputValue = Object.keys(o)
    .filter(function(k) {
      return filter.indexOf(o[k].object.uniqueKey) >= 0;
    })
    .map(function(k) {
      return o[k].object.uniqueKey;
    })
    .filter(function(element, index, array){
      return indexes.indexOf(index) < 0;
    });
  };

  this.save = function() {
    this.currentObject.set(this.attribute, this.inputValue);
    this.initInputValue();
  };

  this.init(currentObject, attribute);
}
ObjectDualViewAttributeListable.prototype = new AttributeListable();
ObjectDualViewAttributeListable.prototype.constructor = ObjectDualViewAttributeListable;


/*
 * AttributeListableSorter Classes
 */
function ColumnAttributeListableSorter(attributeListable) {
  this.attributeListable = attributeListable;
  this.sort = 0;

  this.get = function() {
    var sortIndex = this.sort;
    return this.attributeListable.sort(function(a, b) {
      var one = '' + (a[sortIndex] || "|||");
      var two = '' + (b[sortIndex] || "|||");
      return (one.toLowerCase()) > (two.toLowerCase());
    });
  };
}
